package com.tuoheng.admin.service.airData.query;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.style.column.SimpleColumnWidthStyleStrategy;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tuoheng.admin.constant.SystemConstant;
import com.tuoheng.admin.entity.domain.AirData;
import com.tuoheng.admin.entity.domain.Inspection;
import com.tuoheng.admin.entity.vo.airData.AirDataExcelVo;
import com.tuoheng.admin.enums.AirDataEnum;
import com.tuoheng.admin.enums.MarkEnum;
import com.tuoheng.admin.mapper.AirDataMapper;
import com.tuoheng.admin.mapper.InspectionMapper;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

/**
 * @Author ChengWang
 * @Date 2023/5/24
 */
@Service
@Slf4j
public class QueryAirDataListService {

    @Autowired
    private AirDataMapper airDataMapper;

    @Autowired
    private InspectionMapper inspectionMapper;


    public List<AirData> list(String id) {
        List<AirData> airDataList = airDataMapper.selectList(Wrappers.<AirData>lambdaQuery()
                .eq(AirData::getMark, MarkEnum.VALID.getCode())
                .eq(AirData::getInspectionId, id));

        return airDataList;
    }

    public JsonResult generateTable(String id) {
        //校验
        JsonResult jsonResult = this.check(id);
        if(0 != jsonResult.getCode()){
            log.info("生成excel表格的任务id为空");
            return JsonResult.error(jsonResult.getCode(),jsonResult.getMsg());
        }
        //获取当前任务id下的大气数据
        List<AirData> airDataList = airDataMapper.selectList(Wrappers.<AirData>lambdaQuery()
                .eq(AirData::getMark, MarkEnum.VALID.getCode())
                .eq(AirData::getInspectionId, id));
        if(CollectionUtils.isEmpty(airDataList) || airDataList.size() == 0){
            return JsonResult.error(AirDataEnum.AIR_DATA_IS_NOT_EXIST.getCode(),AirDataEnum.AIR_DATA_IS_NOT_EXIST.getMsg());
        }
        List<AirDataExcelVo> airDataExcelVoList = new ArrayList<>();
        airDataList.forEach(x->{
            AirDataExcelVo vo = new AirDataExcelVo();
            BeanUtils.copyProperties(x,vo);
            airDataExcelVoList.add(vo);
        });
        //获取任务名称
        Inspection inspection = inspectionMapper.selectById(id);
        String name = inspection.getName();
        //获取到vo
        JsonResult result = easyExcel(airDataExcelVoList,name);

        return JsonResult.success("表格上传下载成功");
    }

    public  static JsonResult easyExcel(List<AirDataExcelVo> voArrayList,String inspectionName) {
        String fileNameTemp = "G:/测试/"; //文件输出目录(路径自定义)
        String temp =  UUID.randomUUID().toString()+".xlsx";
        String fileName = fileNameTemp+temp;
        List<List<String>> heads = new ArrayList<>();
        String totalName = "项目名称:"+ inspectionName;
        String[] nameString = SystemConstant.EXCEL_NAME;
        for (String excelName : nameString) {
            heads.add(Arrays.asList(totalName,excelName));
        }
        //需要写出的表格（对应实体类）
        log.info("开始写入表格中");
        EasyExcel.write(fileName, AirDataExcelVo.class).sheet("环境大气数据表").head(heads)
                .registerWriteHandler(new SimpleColumnWidthStyleStrategy(20)) // 简单的列宽策略，列宽20
                //.registerWriteHandler(new SimpleRowHeightStyleStrategy((short)20,(short)20)) // 设置行高策略
                .doWrite(voArrayList);//数据源
        log.info("写入表格已完成");

        return JsonResult.success();
    }

    private JsonResult check(String id) {
        if(StringUtils.isEmpty(id)){
            return JsonResult.error(AirDataEnum.INSPECTION_ID_IS_NULL.getCode(),AirDataEnum.INSPECTION_ID_IS_NULL.getMsg());
        }
        return JsonResult.success();
    }
}
